package jpaSession;

import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.transaction.UserTransaction;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class TestJPASession {

	EntityManagerFactory emf;
	Environment env;
	Logger log;
	PoolingDataSource ds;

	public TestJPASession() {
		PropertyConfigurator.configure("log4j.properties");
		System.setProperty("java.naming.factory.initial",
				"bitronix.tm.jndi.BitronixInitialContextFactory");
		ds = new PoolingDataSource();
		ds.setUniqueName("jdbc/processInstanceDS");
		ds.setClassName("org.h2.jdbcx.JdbcDataSource");
		ds.setMaxPoolSize(3);
		ds.setAllowLocalTransactions(true);
		ds.getDriverProperties().put("user", "sa");
		ds.getDriverProperties().put("password", "sasa");
		ds.getDriverProperties().put("URL", "jdbc:h2:file:/opt/db/h2/test");
		ds.init();
		emf = Persistence
				.createEntityManagerFactory("org.jbpm.persistence.jpa");
		env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
		env.set(EnvironmentName.TRANSACTION_MANAGER,
				TransactionManagerServices.getTransactionManager());
		log = Logger.getLogger(this.getClass());

	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		TestJPASession test=new TestJPASession();
		try {
			test.testLoadSessionWithFactsInEntryPoint();
			System.out.println("testLoadSessionWithFactsInEntryPoint passed");
		} catch (Exception e) {
			System.err.println("testLoadSessionWithFactsInEntryPoint fail");
			e.printStackTrace();
			return;
		}
		
	}
	
	public void testLoadSessionWithFactsInEntryPoint() throws Exception {
		int sessionId;

		UserTransaction ut = (UserTransaction) new InitialContext()
				.lookup("java:comp/UserTransaction");
		ut.begin();
		KnowledgeBase kbase = createKnowledgeBase(EventProcessingOption.STREAM, "jpaSession/cepRules.drl");
		StatefulKnowledgeSession ksession = createKnowledgeSession(
				ClockTypeOption.get("realtime"), kbase);
		sessionId = ksession.getId();
		for (int i = 0; i < 5; i++) {
			ksession.getWorkingMemoryEntryPoint("eventStream").insert("#" + i);
			ksession.fireAllRules();
		}
		ksession.dispose();
		ut.commit();

		log.info("Created session with id=" + sessionId);

		ut = (UserTransaction) new InitialContext()
				.lookup("java:comp/UserTransaction");
		ut.begin();
		ksession = loadKnowledgeSession(sessionId,
				ClockTypeOption.get("realtime"), kbase);
		ksession.dispose();
		ut.commit();
	}
	
	public KnowledgeBase createKnowledgeBase(
			EventProcessingOption processingOption, String... resources) {
		// Create knowledge builder
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		for (String res : resources) {
			if (res.endsWith("bpmn2")) {
				kbuilder.add(ResourceFactory.newClassPathResource(res),
						ResourceType.BPMN2);
			} else if (res.endsWith("drl")) {
				kbuilder.add(ResourceFactory.newClassPathResource(res),
						ResourceType.DRL);
			} else {
				log.error(res + " can not load to knowledge base");
			}
		}
		if (kbuilder.getErrors().size() > 0) {
			for (KnowledgeBuilderError error : kbuilder.getErrors()) {
				log.fatal(error);
			}
			System.exit(0);
		}
		// Create knowledge base
		KnowledgeBaseConfiguration confBase = KnowledgeBaseFactory
				.newKnowledgeBaseConfiguration();
		confBase.setOption(processingOption);
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(confBase);
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	public StatefulKnowledgeSession createKnowledgeSession(
			ClockTypeOption clockType, KnowledgeBase kbase) {
		KnowledgeSessionConfiguration conf = KnowledgeBaseFactory
				.newKnowledgeSessionConfiguration();
		conf.setOption(clockType);
		return JPAKnowledgeService
				.newStatefulKnowledgeSession(kbase, conf, env);
	}

	public StatefulKnowledgeSession loadKnowledgeSession(int id,
			ClockTypeOption clockType, KnowledgeBase kbase) {
		KnowledgeSessionConfiguration conf = KnowledgeBaseFactory
				.newKnowledgeSessionConfiguration();
		conf.setOption(clockType);
		return JPAKnowledgeService.loadStatefulKnowledgeSession(id, kbase,
				conf, env);
	}

}
